/*
 
mDavar 2.0 - Simple Hebrew dictionary for J2ME
Copyright (C) 2007 Josef Planeta <freedavar@centrum.cz>
Copyright (C) 2007 Vladimir Marcenko <vladimir-2@rambler.ru>

 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
*/


import javax.microedition.rms.*;
import javax.microedition.lcdui.*;

import java.io.IOException;
import java.lang.Character;

//format of OPTIONS[] and stored data RECORD[]:

//options[0]	record[0]		remember direction: 0=NO, 1: YES
//options[1]	record[1]		RESERVED
//options[2]	record[2]		for options[0]=1: last direction: 0=FROM HEBREW, 1=TO HEBREW
//options[3]	record[3+4]		RESERVED
//options[4]	record[5+6]		RESERVED

//options[5]:	record[7]		font size: 0=small, 1=medium, 2=large		(index of check)
//options[6]:	record[8]		font style: 0=plain, 1=bold (index of check)
//options[7]:	record[9]		font face: 0=Proportional, 1=Monospaced, 2=System (index of check)

//options[8]:   record[10-12]	0x00RRGGBB and UNSIGNED BYTE values (R+G+B) for background color
//options[9]:   record[13-15]	0x00RRGGBB and UNSIGNED BYTE values (R+G+B) for foreground color
//options[10]:	record[16]		index of predefined background colors. -1 if user defined are used 
//options[11]:	record[17]		index of predefined foreground colors. -1 if user defined are used
//options[12]:  record[18]		hebrew size: 0=small, 1=big, 2=super big

public class Options extends Form implements CommandListener {
    
    Command okCommand = new Command("OK", Command.OK, 1);
    Command cancelCommand = new Command("Cancel", Command.CANCEL, 1);

	
	static public int[] 		options = {	1, //remember direction				//default values
											0, //RESERVED
											0, //last direction
											0, //RESERVED
											0, //RESERVED
											1, //font size (medium)
											0, //font style (plain)  
											0, //font face (proportional)
											0x0000FF00, //back - green 
											0x00000000,	//fore-black OPTION NOT USED YET!
											0,	//back index 0=green 
											0,	//fore index UNUSED YET!
											1};	//0=small he, 1=big he	, 2=super big
    		
	byte[] 		record = new byte[30];				//default values
    
    RecordStore 		recordStore;
    private	 String 	recordStoreName = "mDavar";
    
    TextField tf;

    private	 int 		rgb_values	= 6;
    private int[] 		rgb_defined = {	0x0000ff00,		//green 
    									0x00ff0000,		//red
    									0x0000C0ff,		//blue
    									0x00ffff00,		//yellow
    									0x00ffffff,		//white
    									0x00C0C0C0		//gray
    									};	
    
    
    
//    private mDavar		mD;    
    private ChoiceGroup dictionary, fontFace, fontStyle, fontSize, hebrewSize, back, backown;

    
    public Options() {

    	super("Options");
    	
    	int		f,g;
    	String	s;
    	
        addCommand(okCommand);
    	addCommand(cancelCommand);

    	setCommandListener(this);
        

        loadOptions();
    	   

        dictionary = new ChoiceGroup("Dictionary", ChoiceGroup.MULTIPLE);
        dictionary.append("Remember direction", null);
        dictionary.setSelectedIndex(0, options[0] == 0 ? false : true);
        append(dictionary);
        
        fontSize = new ChoiceGroup("Latin Font size", ChoiceGroup.EXCLUSIVE);
        fontSize.append("Small", null);
        fontSize.append("Medium", null);
        fontSize.append("Large", null);
        fontSize.setSelectedIndex(options[5], true);
        append(fontSize);
        
        fontStyle = new ChoiceGroup("Latin Font style", ChoiceGroup.EXCLUSIVE);
        fontStyle.append("Plain", null);
        fontStyle.append("Bold", null);
        fontStyle.setSelectedIndex(options[6], true);
        append(fontStyle);

        fontFace = new ChoiceGroup("Latin Font face", ChoiceGroup.EXCLUSIVE);
        fontFace.append("Proportional", null);
        fontFace.append("Monospace", null);
        fontFace.append("System", null);
        fontFace.setSelectedIndex(options[7], true);
        append(fontFace);
        

        hebrewSize = new ChoiceGroup("Hebrew size"+'\n'+"(wait after change)", ChoiceGroup.EXCLUSIVE);
        hebrewSize.append("Small", null);
        hebrewSize.append("Big", null);
        hebrewSize.append("Super Big", null);
        hebrewSize.setSelectedIndex(options[12], true);
        append(hebrewSize);

        
        back = new ChoiceGroup("Background color", ChoiceGroup.EXCLUSIVE);
        back.append("Green", null);
        back.append("Red", null);
        back.append("Blue", null);
        back.append("Yellow", null);
        back.append("White", null);
        back.append("Gray", null);

        back.append("User defined RGB", null);
        if(options[10] != -1)
        	back.setSelectedIndex(options[10], true);
        else
        	back.setSelectedIndex(rgb_values, true);		//rgb is last check
        
        append(back);

        s = ((options[8] & 0x00FF0000)>>16) + " "+ ((options[8] & 0x00FF00)>>8) + " " + (options[8] & 0x000000FF);
        
        tf = new TextField("",s,12,0);	//max is: 255,255,255 = 11 chars
        append(tf);
        
    
    }
    
    //-----------------------------------------
    public void loadOptions(){

    	
    	try {
            recordStore = RecordStore.openRecordStore(recordStoreName, true);

            //if there is no record-create with default values!
            if (recordStore.getNumRecords() == 0) {
				// Insert default values
            	options2record();
            	recordStore.addRecord(record, 0, record.length);
            }
            
            //read stored values
            record = recordStore.getRecord(1);

            recordStore.closeRecordStore();
            
        } catch(RecordStoreException rse) { }	            	
    	
        //now convert record[] to options[]
        record2options();
        
        return;

    }
    
    //-----------------------------------------
    
    void saveOptions() {

    	//now prepare record[] according actual options[] data
    	options2record();
    	
    	try {
            
    		recordStore = RecordStore.openRecordStore(recordStoreName, true);

            //because loadOptions() is called BEFORE saveOptions, we can be sure we have 
    		//default values stored.

    		//write record array
            recordStore.setRecord(1, record, 0, record.length);
            
            recordStore.closeRecordStore();
            
        } catch(RecordStoreException rse) { }	            	
    	
        return;
        
    }    
    
   //-----------------------------------------
    //will convert int options to byte[] record 
    void options2record(){
    	

    	record[0] = (byte)options[0];
    	record[1] = (byte)options[1];

    	options[2] = MDD.direction;
    	record[2] = (byte)options[2];
   	
    	record[3] = (byte)(options[3] & 0x000000FF);	//LO
    	record[4] = (byte)(options[3] & 0x0000FF00);	//HI
    	
    	record[5] = (byte)(options[4] & 0x000000FF);	//LO-SIGNED!!!
    	record[6] = (byte)(options[4] & 0x0000FF00);	//HI-SIGNED!!!

    	record[7] = (byte)options[5];
    	record[8] = (byte)options[6];
    	record[9] = (byte)options[7];

    	
    	record[10] = (byte)(int)((options[8] & 0x00FF0000)>>16);	//R-signed
    	record[11] = (byte)(int)((options[8] & 0x0000FF00)>>8);	//G-signed
    	record[12] = (byte)(int)(options[8] & 0x000000FF);	//B-signed


    	record[13] = (byte)(int)((options[9] & 0x00FF0000)>>16);	//R-signed
    	record[14] = (byte)(int)((options[9] & 0x0000FF00)>>8);	//G-signed
    	record[15] = (byte)(int)(options[9] & 0x000000FF);	//B-signed

    	
    	
    	record[16] = (byte)(options[10]);
    	record[17] = (byte)(options[11]);
    	
    	record[18] = (byte)(options[12]);
    	
    	return;
    }
    
   //-----------------------------------------
    
    //will convert record[] to options[]
    void record2options(){

    	int			f;
    	
    	options[0] = record[0];
    	options[1] = record[1];

    	options[2] = record[2];
   	
    	f = (record[3]<0) ? 256 + record[3] : record[3];
    	options[3] = f;									//LO
    	f = (record[4]<0) ? 256 + record[4] : record[4];
    	options[3] += (256 * f);

    	f = (record[5]<0) ? 256 + record[5] : record[5];
    	options[4] = f;									//LO
    	f = (record[6]<0) ? 256 + record[6] : record[6];
    	options[4] += (256 * f);


    	options[5] = record[7];
    	options[6] = record[8];
    	options[7] = record[9];
    	
    	f = (record[10]<0) ? 256 + record[10] : record[10];
    	options[8] = (f << 16);
    	f = (record[11]<0) ? 256 + record[11] : record[11];
    	options[8] += (f << 8);
    	f = (record[12]<0) ? 256 + record[12] : record[12];
    	options[8] += f;

    	f = (record[13]<0) ? 256 + record[13] : record[13];
    	options[9] = (f << 16);
    	f = (record[14]<0) ? 256 + record[14] : record[14];
    	options[9] += (f << 8);
    	f = (record[15]<0) ? 256 + record[15] : record[15];
    	options[9] += f;
    	
    	options[10] = record[16];
    	options[11] = record[17];

    	options[12] = record[18];
    	
    	return;
    }
    
   //-----------------------------------------
    
   
    public static Font getFont() {
        int face;

        switch(options[7]) {
            case 1: face = Font.FACE_MONOSPACE; break;
            case 2: face = Font.FACE_SYSTEM; break;
            default: face = Font.FACE_PROPORTIONAL;
        }
        int style;
        switch(options[6]) {
            case 1: style = Font.STYLE_BOLD; break;
            default: style = Font.STYLE_PLAIN;
        }
        int size;
        switch(options[5]) {
            case 0: size = Font.SIZE_SMALL; break;
            case 2: size = Font.SIZE_LARGE; break;
            default: size = Font.SIZE_MEDIUM;
        }

        return Font.getFont(face, style, size);
        
    }

    //-----------------------------------------
    
    public void commandAction(Command c, Displayable d) {
        
    	String	s,s2;
    	int		f;
    	
    	
    	
    	if(c == okCommand) {

        	//read settings to options variables
        	if(dictionary.isSelected(0))			//direction
                options[0] = 1;
        	else
        		options[0] = 0;

            
            options[5] = (byte)fontSize.getSelectedIndex();
            options[6] = (byte)fontStyle.getSelectedIndex();
            options[7] = (byte)fontFace.getSelectedIndex();

            //background
            options[10] = (byte)back.getSelectedIndex();
            if(options[10] == rgb_values){		//last check ON
            	options[10] = -1;				//last check is rgb == -1: USER DEFINED RGB
            	//now will read RGB string
               	s=tf.getString();	  				//zatim do s - stringu
               	options[8] = getRGB(s);
            }
            else{								//will use predefined values
               	options[8] = rgb_defined[options[10]];            	
            }

            //size hebrew
            options[12] = (byte)hebrewSize.getSelectedIndex();

            saveOptions();
            
        }

    	//FIXED: we must get all images again, but SLOOW-only when we do change
    	if(mDavar.big_he != options[12]){
    		mDavar.big_he = options[12];
    		try {
    			TextGraphics.readImages();				//static
    		} catch (IOException e) {}
    	
    		Search.loadHebrewTable();					//again!
    	}
		
        if(MDD.direction == 0)
        	mDavar.display.setCurrent(mDavar.dict_he);
        else
        	mDavar.display.setCurrent(mDavar.dict_cz);

    
        return;
    }

  
//-------------------------------------------------------------------

public static int getRGB(String s){

	int			f, g;
	String		s2;
	int[]		rgb = {0,0,0};
	
	
	
	f=0;

	//skip no digits
    while(f < s.length() &&  Character.isDigit(s.charAt(f)) != true){
       	f++;
    }
	
    for(g=0; g<3; g++){
    
    	//read color
    	s2="";
    	while(f < s.length() &&  Character.isDigit(s.charAt(f)) == true){
    		s2+=s.charAt(f);
    		f++;
    	}

    	//skip no digits
    	while(f < s.length() &&  Character.isDigit(s.charAt(f)) != true){
    		f++;
    	}

    	rgb[g]=Integer.valueOf(s2).intValue();
    }

    //use g as result
    g = (rgb[0]<<16) + (rgb[1]<<8) + (rgb[2]); 
	
    return g;

}

//-------------------------------------------------------------------

}
