/*
 
mDavar 2.0 - Simple Hebrew dictionary for J2ME
Copyright (C) 2007 Josef Planeta <freedavar@centrum.cz>
Copyright (C) 2007 Vladimir Marcenko <vladimir-2@rambler.ru>

 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
*/


import javax.microedition.lcdui.*;
import java.util.Vector;
import java.io.IOException;

public class TextGraphics {
	
	public static final int HE_CHARS_ALL = 91;		//number of all characters HE
	public static final int HE_VOWELS_ALL = 11;		//number of all vowels HE

	public static int HE_CHARS_HEIGHT = 24;		//He height, initiated, but we will set up according actual HE size
	
	public static final int DISP_BORDER = 14; 		//height of strip bottom, for input HE
	public static final int AVERAGE_CHAR_WIDTH = 9; 	//avrg width... for checking 

	public static Image[] heImg = new Image[HE_CHARS_ALL];
	public static Image[] heVowel = new Image[HE_VOWELS_ALL];
	
	
	static final int HE_CHARS[] =	{
		16, 16, 15, 12, 12, 12, 8, 11};					//[0]=he1.png,..etc. number of chars in each image
														//[7]=he8.png - vowels!
		//IMPORTANT he5/he25/he35 has two one pixel width "empty" characters!
	
//big sized he
	static final int	HEBREW_W[][]=	
	{

		{	
		7,  3,  5,  9,  7, 11,  9,  3,  5,  5,			//small hebrew he1-he7
		7,  7,  3,  6,  4,  5,  7,  5,  7,  7,
		7,  7,  7,  7,  7,  7,  3,  3,  6,  8,
		6,  7,  7,  7,  6,  7,  8,  3,  5,  8,
		8,  3,  8,  6,  7,  8,  8,  3,  5,  7,
		8,  8,  7,  7,  7,  7,  7,  9,  9,  9,
		9,  3,  8,  8,  8,  1,  1,  7,  7,  6,
		7,  8,  5,  6,  8,  8,  5,  6,  7,  8,
		5,  7,  8,  7,  7,  7,  7,  9,  9,  9,
		9 
		},

		{
		10,  4,  7, 13, 11, 18, 14,  4,  7,  7,  9, 12,  4,  8,  4,  8,	//big hebrew he21-he27
		11,  7, 11, 11, 12, 11, 11, 11, 11, 11,  4,  4, 12, 12, 12, 11,	
		12, 10,  9, 10, 11,  4,  7, 11, 11,  4, 11, 10,  9, 11, 11,		//he23
		 4,  7, 11, 11, 11, 10, 10, 10, 10, 10, 13, 12,					//he24
		13, 13,  4,  1,  1, 11, 11, 11, 12, 10,  9, 10,		//!! 2 one pixel empty chars he25
		11,  7,  9, 11, 11,  7, 10,  9, 11,  7, 11, 11,					//he26
		10, 10, 10, 10, 13, 12, 13, 13  								//he27
		},
		
		{
		10, 4,  22, 15, 11, 20, 16,  4,  8,  8, 12, 14,  6,  8,  4, 10,	//super big hebrew he31-he37
		12, 10, 12, 11, 13, 11, 12, 12, 12, 12,  4,  5, 13, 13, 13, 10,
		13, 14, 12, 13, 13,  7,  8, 13, 14,  7, 14, 13, 13, 13, 14,
		 9,  8, 16, 14, 14, 14, 11, 12, 13, 13, 16, 16,
		16, 16,  7,  1,  1, 14, 14, 14, 13, 14, 12, 13,		//!! 2 one pixel empty chars
		13,  7, 10, 13, 14,  7, 13, 13, 14,  8, 16, 14,
		14, 12, 13, 13, 16, 16, 16, 16  
		}
	};

	
	//X offsets and widts in pixels.
	//part 1- consonants he1.png-he7.png
	public static int	hebrew_X[][]=	
	{
		{
		0,  7, 10, 15, 24, 31, 42, 51, 54, 59, 64, 71, 78, 81, 87, 91,	//small he
		0,  7, 12, 19, 26, 33, 40, 47, 54, 61, 68, 71, 74, 80, 88, 94,
		0,  7, 14, 20, 27, 35, 38, 43, 51, 59, 62, 70, 76, 83, 91,
		0,  3,  8, 15, 23, 31, 38, 45, 52, 59, 66, 75,
		0,  9, 18, 21, 29, 37, 45, 46, 47, 54, 61, 67,
		0,  8, 13, 19, 27, 35, 40, 46, 53, 61, 66, 73,
		0,  7, 14, 21, 28, 37, 46,  55
		},

		{
		0,  10, 14, 21, 34, 45, 63, 77, 81,  88,  95, 104, 116, 120, 128, 132, //big he
		0,  11, 18, 29, 40, 52, 63, 74, 85,  96, 107, 111, 115, 127, 139, 151,
		0,  12, 22, 31, 41, 52, 56, 63, 74,  85,  89, 100, 110, 119, 130,
		0,   4, 11, 22, 33, 44, 54, 64, 74,  84,  94, 107,
		0,  13, 26, 30, 31, 32, 43, 54, 65,  77,  87,  96,				//!!
		0,  11, 18, 27, 38, 49, 56, 66, 75,  86,  93, 104,
		0,  10, 20, 30, 40, 53, 65, 78
		},
		
		
		{
		0,  10, 14, 22, 37, 48, 68, 84, 88,  96, 104, 116, 130, 136, 144, 148, //super big he
		0,  12, 22, 34, 45, 58, 69, 81, 93, 105, 117, 121, 126, 139, 152, 165,
		0,  13, 27, 39, 52, 65, 72, 80, 93, 107, 114, 128, 141, 154, 167,
		0,   9, 17, 33, 47, 61, 75, 86, 98, 111, 124, 140,
		0,  16, 32, 39, 40, 41, 55, 69, 83,  96, 110, 122,				//!!
		0,  13, 20, 30, 43, 57, 64, 77, 90, 104, 112, 128,
		0,  14, 26, 39, 52, 68, 84, 100
		}

	
	};
	
	
	//vowels
	public static int	vowel_X[][] ={
		{1,  6, 11, 16, 21, 24, 27, 34, 41, 48, 54 },		//small he
		{1,  6, 11, 16, 21, 24, 27, 34, 41, 48, 55 },		//big he
		{1,  8, 15, 23, 31, 35, 39, 49, 60, 70, 82 }		//super big he
	};
	
	public static int	vowel_Y[][]= 	{
		{3,  3,  3,  3,  3,  3,  3,  3,  3,  3, 13 },		//small he
		{3,  3,  3,  3,  3,  3,  3,  3,  3,  3, 18 },		//big he
		{6,  6,  6,  6,  6,  6,  6,  6,  6,  6, 23 }		//super big he
	};
	public static int	vowel_W[][]= 	{
		{3,  3,  3,  3,  1,  1,  5,  5,  5,  5,  1 },		//small he
		{3,  3,  3,  3,  1,  1,  5,  5,  5,  5,  2 },		//big he
		{8,  7,  8,  8,  4,  4, 10, 11, 10, 11,  2 }		//super big he
	};


	public static int	cholem_correction[] = {0, -1, -1};	//for big_he=0 is 0, for big_he=1 and big_he=2, 
	
	
//---------------------------------------------------------
	
    public static void readImages() throws IOException {

    	Image 		image = null;
//        heImg = new Image[HE_CHARS_ALL];
  //      heVowel = new Image[HE_VOWELS_ALL];
        int 		l = 0;								//left index of image
        int 		r = 0;								//right index of image
        int			f, g;
        Graphics 	gr;									//

        //Needs to be set asap-we will use variable immediatelly!
        if(mDavar.big_he == 0)
    		HE_CHARS_HEIGHT = 14;
    	if(mDavar.big_he == 1)
    		HE_CHARS_HEIGHT = 19;               
    	if(mDavar.big_he == 2)				//super big
    		HE_CHARS_HEIGHT = 24;               

    	
        for(g = 0; g < (HE_CHARS.length-1); g++) {             

        	//use selected image set:
        	if(mDavar.big_he == 0)
        		image = Image.createImage("/he" + (g+1) + ".png");

        	if(mDavar.big_he == 1)        	
        		image = Image.createImage("/he" + "2" + (g+1) + ".png");   //big he     	
    		    		
        	if(mDavar.big_he == 2)        	
        		image = Image.createImage("/he" + "3" + (g+1) + ".png");	//super big he        	

        	
        	//cut image to characters
            r+=HE_CHARS[g];
            
            for( f = l; f < r; f++) {
    			heImg[f] = Image.createImage(HEBREW_W[mDavar.big_he][f], HE_CHARS_HEIGHT);
                gr = heImg[f].getGraphics();
    			gr.drawImage(image, -hebrew_X[mDavar.big_he][f], 0, Graphics.TOP | Graphics.LEFT);
            }
            l = r;

            

        }

        //vowels
    	if(mDavar.big_he == 0)
    		image = Image.createImage("/he8.png");
    	if(mDavar.big_he == 1)
    		image = Image.createImage("/he28.png");        
    	if(mDavar.big_he == 2)
    		image = Image.createImage("/he38.png");        

    	for(f=0; f < HE_CHARS[7]; f++){
			heVowel[f] = Image.createImage(vowel_W[mDavar.big_he][f], vowel_Y[mDavar.big_he][f]);
            gr = heVowel[f].getGraphics();
			gr.drawImage(image, -vowel_X[mDavar.big_he][f], vowel_Y[mDavar.big_he][f], Graphics.BOTTOM | Graphics.LEFT);
		}

    	 
    	
    	    
    }

//-------------------------------------------------------------    
    

    public static Point drawHebrewText(byte[] hebrewText, int len, Graphics graphics, Point start) {  // how new line from right?

        int 			x = start.x;
        int 			y = start.y;
        int				c, v, v2;	//znak(obecne) a vokal + druhy vokal
        int				f, i;
        
        //int imageIndex, v, v2, j;	//v=vocal, v2=second vocal

        for(f=0; f < len; f++){    	
        	c=hebrewText[f]-32;		//index obrazku, pozor, je tam offset 32 ([0] je space!)

//FIXED:id dic data were BAD converted by users...we can have buggy input.
//so, we test also RANGE of input data!
if(c>=HE_CHARS_ALL){		
	continue; 
}
if(c<0 && (c<-127 || c>-117)){	//pouze vokaly: 161az171::-95az-85::-127az-117
	continue;
}
		

        	v2=-1;			//-1 slouzi jako flag: pokud je v2>=0, bude se vykreslovat

        	if(c<0){		//vokal: zkonvertujeme na spravnou hodnotu,
        		v=c+127;	//do promenne vokalu+korekce

        		//nacteme dalsi. POZOR-TEZKY BUG: muze byt zase vokal! Protoze se vokaly prepisuji, i u Davaru,
        		//jedna se nejspis o 'o'. Vice nez dva vokaly nepredpokladame (osetri se to pri exportu do mdd)
        		f++;		//DOCELA DRSNE menit f v cyklu!!!

if(f>=len) break;

				c=hebrewText[f]-32;		//index obrazku, pozor, je tam offset 32 ([0] je space!)	

        		if(c<0){		//POZOR-druhy vokal. Vice NENI pripustno a bude osetreno v exportu do mdd!
        			v2=c+127;	//do promenne vokalu+korekce
        			f++;		//DOCELA DRSNE menit f v cyklu!!!
if(f>=len) break;

					c=hebrewText[f]-32;		//index obrazku, pozor, je tam offset 32 ([0] je space!)	
        			//ted ignorujeme zbyvajici vokaly
//				while(((MainClass.inBuf_HE[f]-32)<0) && (f < MainClass.rec_length_HE))
//					f++;
        		}

        		//vykreslit konsonant
        		graphics.drawImage(heImg[c], x, y, Graphics.TOP | Graphics.LEFT); 

        		//vykreslit vokal
        		i=0; 	//zatim zadny posun
        		//CENTRUJEME VOKAL v, ale NE pro "o"
        		if(v != 10){
        			i=(HEBREW_W[mDavar.big_he][c]-vowel_W[mDavar.big_he][v])/2;
        			i=(i>=0) ? i : 0;
        		}
        		else{
        			i= cholem_correction[mDavar.big_he];
        		}
        		graphics.drawImage(heVowel[v], x+i, y+HE_CHARS_HEIGHT, Graphics.BOTTOM | Graphics.LEFT);

        		//CENTRUJEME VOKAL v2-pokud byl (>=0), ale NE pro "o"
        		i=0; 			//zatim zadny posun
        		if(v2 >= 0){	//BYL druhy vokal
        			i=(HEBREW_W[mDavar.big_he][c]-vowel_W[mDavar.big_he][v2])/2;
        			i=(i>=0) ? i : 0;
        			i=(v2 == 10) ? 0 : i;	//pro 'o' je posun 0
        			graphics.drawImage(heVowel[v2], x+i, y+HE_CHARS_HEIGHT, Graphics.BOTTOM | Graphics.LEFT); 
        		}

        		x+=HEBREW_W[mDavar.big_he][c];

        	}
        	else{			//nebyl vokal
        		graphics.drawImage(heImg[c], x, y, Graphics.TOP | Graphics.LEFT); 
        		x += HEBREW_W[mDavar.big_he][c];
        	}

        	//test, zda nejsme na konci
        	if(x + AVERAGE_CHAR_WIDTH > graphics.getClipWidth()){	//AVERAGE_CHAR_WIDTH 9 je prumerna max. sirka he znaku
        		//novy radek
        		y += (graphics.getFont().getHeight() > HE_CHARS_HEIGHT) ? graphics.getFont().getHeight():HE_CHARS_HEIGHT;
        		x=0;
        	}
    
        }
        return new Point(x,  y);
    }
    
    
    
//----------------------------------------------------------------------------------
    
    //Note: we use len-1 to avoid conversion+cripple final \0 in translationText
    
    public static Point drawTranslationText(byte[] translationText, int len, Graphics graphics, Point start) {

    	
    	int l = 0, r = l, f, x = start.x, y = start.y;
        int fontHeight = graphics.getFont().getHeight();
        String s = new String(translationText); // convert to unicode?
        //convert to UNICODE
        s = MDD.Convert2Unicode(s, len-1); //prekodujeme
        
        while(r < len-1) {
            f = graphics.getFont().stringWidth(s.substring(l, r));
            if((x+f+AVERAGE_CHAR_WIDTH) < graphics.getClipWidth())	//ok, vlezeme se
            	r++;
            else{					//nevlezeme se
                graphics.drawString(s.substring(l,r), x, y, Graphics.TOP|Graphics.LEFT); 
                l=r;
                r++;
                y+=(fontHeight > HE_CHARS_HEIGHT) ? fontHeight:HE_CHARS_HEIGHT;
                x=0;
            }
        }
        graphics.drawString(s.substring(l,r), x, y, Graphics.TOP|Graphics.LEFT); 
        x += graphics.getFont().stringWidth(s.substring(l, r));
        return new Point(x,  y);
    }    

  //----------------------------------------------------------------//
    
    
}
